@@ -2,46 +2,52 @@
 # SPDX-License-Identifier: MPL-2.0+
 
 # Master user-exposed function
-minimaxApprox <- function(fn, lower, upper, degree, relErr = FALSE, xi = NULL,
-                          opts = list()) {
+minimaxApprox <- function(fn, lower, upper, degree, relErr = FALSE,
+                          basis = "Chebyshev", xi = NULL, opts = list()) {
+
+  basis <- tolower(substr(basis, 1L, 1L))
+  if (!(basis %in% c("c", "m"))) {
+    stop("Must select either 'C'hebyshev or 'm'onomial basis for analysis.")
+  }
 
   # Handle configuration options
-  if (!("maxiter" %in% names(opts))) {
+  nopts <- names(opts)
+  if (!("maxiter" %in% nopts)) {
     opts$maxiter <- 100L
   }
 
-  if (!("miniter" %in% names(opts))) {
+  if (!("miniter" %in% nopts)) {
     opts$miniter <- 10L
   }
 
-  if ("conviter" %in% names(opts)) {
+  if ("conviter" %in% nopts) {
     # If actually passed then overwrite maxiter if conviter > maxiter.
     opts$maxiter <- max(opts$maxiter, opts$conviter)
   } else {
     opts$conviter <- 30L
   }
 
-  if (!("showProgress" %in% names(opts))) {
+  if (!("showProgress" %in% nopts)) {
     opts$showProgress <- FALSE
   }
 
-  if (!("convrat" %in% names(opts))) {
+  if (!("convrat" %in% nopts)) {
     # Using 1 + 1e-9 - See Cody (1968) page 250. Can reasonably expect between
     # 9 & 12 significant figures.
     opts$convrat <- 1.000000001
   }
 
-  if (!("tol" %in% names(opts))) {
+  if (!("tol" %in% nopts)) {
     opts$tol <- 1e-14
   }
 
   # Used for cases where we check polynomial degree n + 1.
   # See issue 2 https://github.com/aadler/minimaxApprox/issues/2
-  if (!("tailtol" %in% names(opts))) {
+  if (!("tailtol" %in% nopts)) {
     opts$tailtol <- min(1e-10, (upper - lower) / 1e6)
   }
 
-  if (!("ztol" %in% names(opts))) {
+  if (!("ztol" %in% nopts)) {
     opts$ztol <- NULL
   }
 
@@ -51,15 +57,15 @@ minimaxApprox <- function(fn, lower, upper, degree, relErr = FALSE, xi = NULL,
   }
 
   if (any(degree < 0) || any(floor(degree) < degree)) {
-    stop("Polynomial degrees must be integers of least 0 (constant).")
+    stop("Degrees must be integers of least 0 (constant).")
   }
 
-  if (length(degree) == 2L) {        # Rational approximation requested
+  if (length(degree) == 2L) {         # Rational approximation requested
     numerd <- as.integer(degree[1L])
     denomd <- as.integer(degree[2L])
     ratApprox <- TRUE
   } else if (length(degree) == 1L) {
-    ratApprox <- FALSE               # Polynomial approximation requested
+    ratApprox <- FALSE                # Polynomial approximation requested
     if (!is.null(xi)) {
       message("Polynomial approximation uses Chebyshev nodes for initial ",
               "guess. Any passed xi is ignored.")
@@ -73,9 +79,9 @@ minimaxApprox <- function(fn, lower, upper, degree, relErr = FALSE, xi = NULL,
 
   # Call Calculation Functions
   mmA <- if (ratApprox) {
-    remRat(fn, lower, upper, numerd, denomd, relErr, xi, opts)
+    remRat(fn, lower, upper, numerd, denomd, relErr, basis, xi, opts)
   } else {
-    tryCatch(remPoly(fn, lower, upper, as.integer(degree), relErr, opts),
+    tryCatch(remPoly(fn, lower, upper, as.integer(degree), relErr, basis, opts),
              error = function(e) simpleError(trimws(e$message)))
   }
 
@@ -86,10 +92,10 @@ minimaxApprox <- function(fn, lower, upper, degree, relErr = FALSE, xi = NULL,
   # resulting degree n and message appropriately.
 
   if (!ratApprox && inherits(mmA, "simpleError")) {
-    if (grepl("singular", mmA$message, fixed = TRUE)) { # May be different error
-      if (!is.null(opts$tailtol)) {
+    if (grepl("singular", mmA$message, fixed = TRUE)) { # nolint unnecessary_nested_if_linter
+      if (!is.null(opts$tailtol)) { # May be different error
         mmA <- tryCatch(remPoly(fn, lower, upper, as.integer(degree + 1L),
-                                relErr, opts),
+                                relErr, basis, opts),
                         error = function(e) simpleError(trimws(e$message)))
         if (inherits(mmA, "simpleError")) {
           stop("The algorithm neither converged when looking for a polynomial",
@@ -107,7 +113,7 @@ minimaxApprox <- function(fn, lower, upper, degree, relErr = FALSE, xi = NULL,
                           "option. The result is a polynomial of length",
                           degree, "as the uppermost coefficient is effectively",
                           "0.")
-            mmA$a <- mmA$a[-length(mmA$a)]
+            mmA$a <- mmA$a[-n]
             message(mess)
           } else {
             stop("The algorithm did not converge when looking for a polynomial",
@@ -120,8 +126,6 @@ minimaxApprox <- function(fn, lower, upper, degree, relErr = FALSE, xi = NULL,
         stop("The algorithm did not converge when looking for a polynomial of ",
              "degree ", degree, " and NULL was passed to the tailtol option.")
       }
-    } else {
-      stop(mmA$message)
     }
   }
 
@@ -139,7 +143,7 @@ minimaxApprox <- function(fn, lower, upper, degree, relErr = FALSE, xi = NULL,
   if (mmA$unchanged && !mmA$converged) {
     warning("Convergence to requested ratio and tolerance not achieved in ",
             mmA$i, " iterations.\n", mmA$unchanging_i, " successive ",
-            "calculated errors were too close to each other to warrant ",
+            "calculated solutions were too close to each other to warrant ",
             "further iterations.\nThe ratio is ",
             fC(mmA$mxae / mmA$expe, d = 14L),
             " times expected and the difference is ",
@@ -160,11 +164,30 @@ minimaxApprox <- function(fn, lower, upper, degree, relErr = FALSE, xi = NULL,
     gotWarning <- TRUE
   }
 
-  coefficients <- if (ratApprox) list(a = mmA$a, b = mmA$b) else list(a = mmA$a)
+  coefficients <- if (ratApprox) {
+    list(a = mmA$a, b = mmA$b)
+  } else {
+    list(a = mmA$a)
+  }
+
+  if (basis == "m") {
+    monomialEq <- NULL
+    polynomalBasis <- "Monomial"
+  } else {
+    monomialEq <- list(aMono = cheb2mon(mmA$a))
+    polynomalBasis <- "Chebyshev"
+    if (ratApprox) {
+      monomialEq <- c(monomialEq, list(bMono = cheb2mon(mmA$b)))
+      monomialEq <- mapply(`/`, monomialEq, monomialEq$bMono[1L],
+                           SIMPLIFY = FALSE)
+    }
+  }
+
   diagnostics <- list(ExpErr = mmA$expe, ObsErr = mmA$mxae, iterations = mmA$i,
-                      Basis = mmA$x, Warning = gotWarning)
-  ret <- c(coefficients, diagnostics)
+                      Extrema = mmA$x, Warning = gotWarning)
+  ret <- c(coefficients, monomialEq, diagnostics)
   attr(ret, "type") <- if (ratApprox) "Rational" else "Polynomial"
+  attr(ret, "basis") <- polynomalBasis
   attr(ret, "func") <- fn
   attr(ret, "range") <- c(lower, upper)
   attr(ret, "relErr") <- relErr
@@ -177,18 +200,43 @@ minimaxApprox <- function(fn, lower, upper, degree, relErr = FALSE, xi = NULL,
 
 # Evaluation convenience function. Identical to evalFunc but tests for
 # inheritance from minimaxApprox.
-minimaxEval <- function(x, mmA) {
+minimaxEval <- function(x, mmA, basis = "Chebyshev") {
   if (!inherits(mmA, "minimaxApprox")) {
     stop("This function only works with 'minimaxApprox' objects.")
   }
-  evalFunc(x, mmA)
+  requestedbasis <- tolower(substr(basis, 1L, 1L))
+  onlyMono <- attr(mmA, "basis") == "Monomial"
+  if (!(requestedbasis %in% c("c", "m"))) {
+    stop("Select either the 'M'onomial or 'C'hebyshev basis.")
+  }
+  if (requestedbasis == "c") {
+    if (onlyMono) {
+      message("Analysis was run using only the monomial basis. Calculating ",
+              "errors using monomials.")
+      evalFunc(x, mmA, "m")
+    } else {
+      evalFunc(x, mmA, "c")
+    }
+  } else {
+    if (onlyMono) {
+      evalFunc(x, mmA, "m")
+    } else {
+      RR <- list(a = mmA$aMono)
+      if ("bMono" %in% names(mmA)) RR <- c(RR, list(b = mmA$bMono))
+      evalFunc(x, RR, "m")
+    }
+  }
 }
 
 # Minimax approximation error convenience function. Based on remErr but takes a
-# completed mmA object.
+# completed mmA object with relErr and basis as attributes.
 minimaxErr <- function(x, mmA) {
   if (!inherits(mmA, "minimaxApprox")) {
     stop("This function only works with 'minimaxApprox' objects.")
   }
-  remErr(x, mmA, attr(mmA, "func"), attr(mmA, "relErr"))
+  y <- callFun(attr(mmA, "func"), x)
+  ret <- evalFunc(x, mmA, tolower(substr(attr(mmA, "basis"), 1L, 1L))) - y
+  if (attr(mmA, "relErr")) ret <- ret / y
+
+  ret
 }
