@@ -12,12 +12,6 @@ chebNodes <- function(n, a, b) {
   sort(0.5 * (a + b + (b - a) * cos((2 * seq_len(n) - 1) * pi / (2 * n))))
 }
 
-# Create Vandermonde matrix to a polynomial degree n, or n + 1 terms.
-vanderMat <- function(x, n) {
-  np1 <- n + 1L
-  matrix(rep(x, each = np1) ^ (seq_len(np1) - 1L), ncol = np1, byrow = TRUE)
-}
-
 # Call the function being approximated on the points x.
 callFun <- function(fn, x) {
   if (!is.function(fn)) stop("Unable to parse function.")
@@ -27,40 +21,35 @@ callFun <- function(fn, x) {
 # Check that the values passed are oscillating in sign
 isOscil <- function(x) all(abs(diff(sign(x))) == 2)
 
-# polyCalc uses a Compensated Horner Method based on Langlois et al.(2006)
-# https://drops.dagstuhl.de/opus/volltexte/2006/442/
-# As primary bottleneck, it was ported to C for speed.
-polyCalc <- function(x, a) {
-  .Call(compHorner_c, as.double(x), as.double(a))
-}
-
-# Function to calculate value of minimax approximation at x given a & b.
-evalFunc <- function(x, R) {
-  ret <- polyCalc(x, R$a)
+evalFunc <- function(x, R, basis) {
+  calcFunc <- switch(EXPR = basis, m = polyCalc, chebCalc)
+  ret <- calcFunc(x, R$a)
   if ("b" %in% names(R)) {
-    ret <- ret / polyCalc(x, R$b)
+    ret <- ret / calcFunc(x, R$b)
   }
+
   ret
 }
 
 # Function to calculate error between known and calculated values.
-remErr <- function(x, R, fn, relErr) {
+remErr <- function(x, R, fn, relErr, basis) {
   if (relErr) {
     y <- callFun(fn, x)
-    (evalFunc(x, R) - y) / y
+    (evalFunc(x, R, basis) - y) / y
   } else {
-    evalFunc(x, R) - callFun(fn, x)
+    evalFunc(x, R, basis) - callFun(fn, x)
   }
 }
 
 # Function to identify roots of the error equation for use as bounds in finding
 # the maxima and minima.
-findRoots <- function(x, R, fn, relErr) {
+findRoots <- function(x, R, fn, relErr, basis) {
   r <- double(length(x) - 1L)
   for (i in seq_along(r)) {
     intv <- c(x[i], x[i + 1L])
     root <- tryCatch(uniroot(remErr, interval = intv, extendInt = "no", R = R,
-                             fn = fn, relErr = relErr),
+                             fn = fn, relErr = relErr, basis = basis,
+                             tol = sqrt(.Machine$double.eps)),
                      error = function(cond) simpleError(trimws(cond$message)))
 
     # If there is no root in the interval, take the endpoint closest to zero.
@@ -70,27 +59,30 @@ findRoots <- function(x, R, fn, relErr) {
       r[i] <- root$root
     }
   }
+
   r
 }
 
 # Function to identify new x positions. This algorithm uses the multi-switch
 # paradigm, not the single switch.
-switchX <- function(r, l, u, R, fn, relErr) {
+switchX <- function(r, l, u, R, fn, relErr, basis) {
   bottoms <- c(l, r)
   tops <- c(r, u)
   x <- double(length(bottoms))
   attr(x, "ZeroBasis") <- FALSE
-  maximize <- sign(remErr(l, R, fn, relErr)) == 1
+  maximize <- sign(remErr(l, R, fn, relErr, basis)) == 1
   for (i in seq_along(x)) {
     intv <- c(bottoms[i], tops[i])
+    # Tighter tolerances than the default lead to issues (AA: 2024-01-31).
     extrma <- tryCatch(optimize(remErr, interval = intv, R = R, fn = fn,
-                                relErr = relErr, maximum = maximize),
+                                relErr = relErr, basis = basis,
+                                maximum = maximize),
                        error = function(cond) simpleError(trimws(cond$message)))
 
     # If no extremum then the take endpoint with "better" value depending if we
     # are maximizing or minimizing.
     if (inherits(extrma, "simpleError")) {
-      endPtErr <- remErr(intv, R, fn, relErr)
+      endPtErr <- remErr(intv, R, fn, relErr, basis)
       if (maximize) {
         x[i] <- intv[which.max(endPtErr)]
       } else {
@@ -102,7 +94,7 @@ switchX <- function(r, l, u, R, fn, relErr) {
 
     # Test endpoints for max/min even if an extremum was found.
     p <- c(bottoms[i], x[i], tops[i])
-    E <- remErr(p, R, fn, relErr)
+    E <- remErr(p, R, fn, relErr, basis)
 
     if (maximize) {
       x[i] <- p[which.max(E)]
@@ -113,12 +105,13 @@ switchX <- function(r, l, u, R, fn, relErr) {
     # Test for 0 value at function if relative error
     if (relErr && callFun(fn, x[i]) == 0) {
       attr(x, "ZeroBasis") <- TRUE
+      peturb <- 1e-12
       if (x[i] == l) {
-        x[i] <- x[i] + 1e-12
+        x[i] <- x[i] + peturb
       } else if (x[i] == u) {
-        x[i] <- x[i] - 1e-12
+        x[i] <- x[i] - peturb
       } else {
-        xreplace <- c(x[i] - 1e-12, x[i] + 1e-12)
+        xreplace <- c(x[i] - peturb, x[i] + peturb)
         fnreplace <- callFun(fn, xreplace)
         if (maximize) {
           x[i] <- xreplace[which.max(fnreplace)]
@@ -131,6 +124,7 @@ switchX <- function(r, l, u, R, fn, relErr) {
     # Flip maximize.
     maximize <- !maximize
   }
+
   x
 }
 
@@ -154,9 +148,24 @@ isConverged <- function(errs, expe, convrat, tol) {
   isOscil(errs) && errDistance && errMagnitude
 }
 
+isUnchanging <- function(errs, errs_last, convrat, tol) {
+  denomProblem <- which(errs_last == 0)
+  # If any are actually 0, then perturb them by 1e-12. Ratio becomes 1 and
+  # difference remains 0.
+  if (length(denomProblem) > 0L) {
+    errs[denomProblem] <- errs[denomProblem] + 1e-12
+    errs_last[denomProblem] <- errs_last[denomProblem] + 1e-12
+  }
+  errsDiff <- abs(errs - errs_last)
+  all(abs(errs / errs_last) <= convrat) ||
+    (all(errsDiff <= tol) && all(errsDiff > .Machine$double.eps))
+}
+
 # Check denominator polynomial for zero in the requested range.
-checkDenom <- function(a, l, u) {
-  dngrRt <- tryCatch(uniroot(polyCalc, c(l, u), extendInt = "no", a = a),
+checkDenom <- function(a, l, u, basis) {
+  calcFn <- if (basis == "m") polyCalc else chebCalc
+  dngrRt <- tryCatch(uniroot(calcFn, c(l, u), extendInt = "no", a = a,
+                             tol = .Machine$double.eps),
                      error = function(cond) simpleError(trimws(cond$message)))
   if (inherits(dngrRt, "simpleError")) {
     return(NULL)
@@ -167,11 +176,11 @@ checkDenom <- function(a, l, u) {
 
 # Check for coefficient irrelevancy.
 checkIrrelevant <- function(a, l, u, zt) {
-  if (!is.null(zt) && length(a) > 0) {
+  n <- length(a)
+  if (!is.null(zt) && n > 0) {
     xmax <- max(abs(l), abs(u))
-    for (i in seq_along(a)) {
-      if (abs(a[i] * xmax ^ (i - 1L)) <= zt) a[i] <- 0
-    }
+    a <- ifelse(abs(a * xmax^(seq_len(n) - 1L)) <= zt, 0, a)
   }
+
   a
 }
